@@ -53,19 +53,19 @@ class Command(object):
 			return callback(*args, **kwargs)
 		except ScalrAPIError, e:
 			print e
-			sys.exit()	
+			sys.exit(1)
 	
 	def pretty(self, callback, *args, **kwargs):
 		try:
 			return TableViewer(callback(*args, **kwargs))
 		except ScalrAPIError, e:
 			return e
-			sys.exit()
+			sys.exit(1)
 			
 	def require(self, *args):
 		if not all(args):
 			print self.parser.format_help(TitledHelpFormatter())
-			sys.exit()
+			sys.exit(1)
 	
 	@classmethod
 	def inject_options(cls, parser):
@@ -90,6 +90,25 @@ class Command(object):
 					logger=self.config.logger)
 		return conn
 
+
+class FarmRoleUpdateParameterValue(Command):
+	name = 'update-farm-role-parameter'
+	help = '-r farm-role-id -n param-name -v param-value'
+
+	def __init__(self, config, *args):
+		super(FarmRoleUpdateParameterValue, self).__init__(config, *args)
+		self.require(self.options.farm_role_id, self.options.param_name, self.options.param_value)
+
+	@classmethod
+	def inject_options(cls, parser):
+		parser.add_option("-r", "--farm-role-id", dest="farm_role_id", help="FarmRoleID")
+		parser.add_option("-n", "--param-name", dest="param_name", help="Parameter Name")
+		parser.add_option("-v", "--value", dest="param_value", help="Parameter Value")
+
+	def run(self):
+		args = (self.options.farm_role_id, self.options.param_name, self.options.param_value)
+		print self.pretty(self.connection.update_farm_role_parameter, *args)
+
 		
 class ApacheVhostsList(Command):
 	name = 'list-apache-virtual-hosts'
@@ -155,7 +174,7 @@ class DmDeployApplication(Command):
 
 	@classmethod
 	def inject_options(cls, parser):
-		app_name_help = "The name of application could be used INSTEAD of ID. Use name %s to test deployment process on your role." % cls.test_app
+		app_name_help = "The name of application can be used INSTEAD of ID. Use name %s to test deployment process on your role." % cls.test_app
 		parser.add_option("-n", "--app-name", dest="app_name", default=None, help=app_name_help)
 		parser.add_option("-a", "--app-id", dest="app_id", help="Application ID")
 		parser.add_option("-r", "--farm-role-id", dest="farm_role_id", help="FarmRole ID")
@@ -192,6 +211,9 @@ class DmDeployApplication(Command):
 		tasks = self.call_api_method(self.connection.dm_deploy_application, *args)
 		print TableViewer(tasks)
 							
+
+		failed_tasks = []
+		done_tasks = []
 		#check task status in loop
 		if not self.options.non_interactive and not isinstance(tasks,ScalrAPIError) and tasks:
 			for task in tasks:
@@ -209,15 +231,23 @@ class DmDeployApplication(Command):
 							status = ts_list[0].status
 						
 							if status == 'deployed':
-								print "Application '%' has been successfully deployed on server %s." % (self.options.app_name, task.server_id)
+								task.status = 'deployed' #changing old status
+								done_tasks.append(task)
+								print "Application '%s' has been successfully deployed on server %s." % (self.options.app_name, task.server_id)
 								break
 							
 							elif status == 'failed':
+								task.status = 'failed' #changing old status
+								failed_tasks.append(task) #task
 								#show log
 								try:
-									print self.pretty(self.connection.dm_get_deployment_task_log,task.task_id)
+									log = self.connection.dm_get_deployment_task_log(task.task_id)
+									print TableViewer(log)
+
+									if log.scalr_objects:
+										task.errmsg = log.scalr_objects[0].message
 								finally:
-									print 'Deployment process has failed.'
+									print 'Deployment process %s has failed on server %s.' % (task.task_id, task.server_id)
 								break
 							
 							elif status == 'pending':
@@ -230,10 +260,22 @@ class DmDeployApplication(Command):
 						time.sleep(attempt)
 						
 					except (KeyboardInterrupt, SystemExit):
+						#stops cheking current server and proceeds to the next
 						break
 				else:
 					print "Maximum number of attempts was reached. On server %s application has not been deployed." % task.server_id
 				print ''
+
+			if done_tasks:
+				print 'Completed tasks:'
+				print TableViewer(done_tasks)
+
+			if failed_tasks:
+				print 'Failed tasks:'
+				print TableViewer(failed_tasks)
+				sys.exit(1)
+			else:
+				print 'All deployment tasks have finished successfully.'
 				
 		
 class DmDeployApplicationAlias(DmDeployApplication):
@@ -365,10 +407,10 @@ class EventsList(Command):
 class LogsList(Command):
 	name = 'list-logs'
 	help = '{-f farm-id | -n name} [-i server-id -s start-from -l limit]'
-	
+
 	def __init__(self, config, *args):
 		super(LogsList, self).__init__(config, *args)
-		self.require(self.options.farm_id)
+		self.require(self.options.farm_id or self.options.farm_name)
 
 	@classmethod
 	def inject_options(cls, parser):
@@ -377,12 +419,33 @@ class LogsList(Command):
 		parser.add_option("-i", "--server-id", dest="server_id", default=None, help="Instance ID")
 		parser.add_option("-s", "--start-from", dest="start", default=None, help="Start from specified event number (Can be used for paging)")
 		parser.add_option("-l", "--record-limit", dest="limit", default=None, help="Limit number of returned events (Can be used for paging)")
-	
+
 	def run(self):
 		args = (self.options.farm_id, self.options.server_id, self.options.start, self.options.limit)
 		print self.pretty(self.connection.list_logs, *args)
 
 
+class ScriptingLogsList(Command):
+	name = 'list-scripting-logs'
+	help = '{-f farm-id | -n name} [-i server-id -s start-from -l limit]'
+
+	def __init__(self, config, *args):
+		super(ScriptingLogsList, self).__init__(config, *args)
+		self.require(self.options.farm_id or self.options.farm_name)
+
+	@classmethod
+	def inject_options(cls, parser):
+		parser.add_option("-f", "--farm-id", dest="farm_id", default=None, help="Farm ID")
+		parser.add_option("-n", "--farm-name", dest="farm_name", default=None, help="The name of farm could be used INSTEAD of ID")
+		parser.add_option("-i", "--server-id", dest="server_id", default=None, help="Instance ID")
+		parser.add_option("-s", "--start-from", dest="start", default=None, help="Start from specified event number (Can be used for paging)")
+		parser.add_option("-l", "--record-limit", dest="limit", default=None, help="Limit number of returned events (Can be used for paging)")
+
+	def run(self):
+		args = (self.options.farm_id, self.options.server_id, self.options.start, self.options.limit)
+		print self.pretty(self.connection.scripting_list_logs, *args)
+
+
 class RolesList(Command):
 	name = 'list-roles'
 	help = '[-p platform -n name -x prefix -i image-id]'
@@ -797,13 +860,13 @@ class ApacheVhostCreate(Command):
 			
 			if not os.path.exists(self.options.cert_path):
 				print '%s : file with certificate not found.' % self.options.cert_path
-				sys.exit()
+				sys.exit(1)
 			else: 
 				cert = open(self.options.cert_path, 'r').read()
 				
 			if not os.path.exists(self.options.pk_path):
 				print '%s : file with private key not found.' % self.options.pk_path
-				sys.exit()
+				sys.exit(1)
 			else: 
 				pk = open(self.options.pk_path, 'r').read()
 				
@@ -843,16 +906,26 @@ class EnvironmentsList(Command):
 
 	def run(self):
 		print self.pretty(self.connection.list_environments)
-		
-		
+
+
 class ShowConfig(Command):
 	name = 'show-config'
-		
-	def run(self):		
+
+	def run(self):
 		e = Environment.from_ini(self.config.base_path)
 		print e
-		
-		
+
+
+class CheckUpdates(Command):
+	name = 'check-updates'
+
+	def run(self):
+		import pkg_resources
+		working_set = pkg_resources.working_set
+		versions = dict([(d.key, d.version) for d in working_set])
+		print 'Current version is: %s' % (versions['scalr'])
+
+
 def waits(n):
 	num = 0
 	sum = 0
